# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'C:/bld/qgis_1553641778225/work/python/plugins/db_manager/ui/DlgQueryBuilder.ui'
#
# Created by: PyQt5 UI code generator 5.6
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_DbManagerQueryBuilderDlg(object):
    def setupUi(self, DbManagerQueryBuilderDlg):
        DbManagerQueryBuilderDlg.setObjectName("DbManagerQueryBuilderDlg")
        DbManagerQueryBuilderDlg.resize(797, 463)
        self.verticalLayout_7 = QtWidgets.QVBoxLayout(DbManagerQueryBuilderDlg)
        self.verticalLayout_7.setObjectName("verticalLayout_7")
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setSizeConstraint(QtWidgets.QLayout.SetDefaultConstraint)
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.formLayout = QtWidgets.QFormLayout()
        self.formLayout.setFieldGrowthPolicy(QtWidgets.QFormLayout.AllNonFixedFieldsGrow)
        self.formLayout.setLabelAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.formLayout.setObjectName("formLayout")
        self.label = QtWidgets.QLabel(DbManagerQueryBuilderDlg)
        self.label.setObjectName("label")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.label)
        self.col = QtWidgets.QTextEdit(DbManagerQueryBuilderDlg)
        self.col.setMinimumSize(QtCore.QSize(400, 0))
        self.col.setObjectName("col")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.col)
        self.label_2 = QtWidgets.QLabel(DbManagerQueryBuilderDlg)
        self.label_2.setObjectName("label_2")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.label_2)
        self.tab = QtWidgets.QLineEdit(DbManagerQueryBuilderDlg)
        self.tab.setFrame(True)
        self.tab.setObjectName("tab")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.tab)
        self.where = QtWidgets.QTextEdit(DbManagerQueryBuilderDlg)
        self.where.setObjectName("where")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.where)
        self.group = QtWidgets.QTextEdit(DbManagerQueryBuilderDlg)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.group.sizePolicy().hasHeightForWidth())
        self.group.setSizePolicy(sizePolicy)
        self.group.setMaximumSize(QtCore.QSize(16777215, 25))
        self.group.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.group.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.group.setObjectName("group")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.FieldRole, self.group)
        self.order = QtWidgets.QTextEdit(DbManagerQueryBuilderDlg)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.order.sizePolicy().hasHeightForWidth())
        self.order.setSizePolicy(sizePolicy)
        self.order.setMaximumSize(QtCore.QSize(16777215, 25))
        self.order.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.order.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.order.setObjectName("order")
        self.formLayout.setWidget(4, QtWidgets.QFormLayout.FieldRole, self.order)
        self.label_3 = QtWidgets.QLabel(DbManagerQueryBuilderDlg)
        self.label_3.setObjectName("label_3")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.label_3)
        self.label_4 = QtWidgets.QLabel(DbManagerQueryBuilderDlg)
        self.label_4.setObjectName("label_4")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.LabelRole, self.label_4)
        self.label_5 = QtWidgets.QLabel(DbManagerQueryBuilderDlg)
        self.label_5.setObjectName("label_5")
        self.formLayout.setWidget(4, QtWidgets.QFormLayout.LabelRole, self.label_5)
        self.horizontalLayout_6.addLayout(self.formLayout)
        self.verticalLayout_4 = QtWidgets.QVBoxLayout()
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.toolBox = QtWidgets.QToolBox(DbManagerQueryBuilderDlg)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.toolBox.sizePolicy().hasHeightForWidth())
        self.toolBox.setSizePolicy(sizePolicy)
        self.toolBox.setMinimumSize(QtCore.QSize(250, 0))
        self.toolBox.setMaximumSize(QtCore.QSize(250, 16777215))
        self.toolBox.setObjectName("toolBox")
        self.page = QtWidgets.QWidget()
        self.page.setGeometry(QtCore.QRect(0, 0, 250, 291))
        self.page.setObjectName("page")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.page)
        self.verticalLayout_5.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.checkBox = QtWidgets.QCheckBox(self.page)
        font = QtGui.QFont()
        font.setPointSize(8)
        font.setKerning(True)
        self.checkBox.setFont(font)
        self.checkBox.setObjectName("checkBox")
        self.verticalLayout_5.addWidget(self.checkBox)
        self.tables = QtWidgets.QComboBox(self.page)
        self.tables.setObjectName("tables")
        self.tables.addItem("")
        self.verticalLayout_5.addWidget(self.tables)
        self.columns = QtWidgets.QComboBox(self.page)
        self.columns.setObjectName("columns")
        self.columns.addItem("")
        self.verticalLayout_5.addWidget(self.columns)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_5.addItem(spacerItem)
        self.line_2 = QtWidgets.QFrame(self.page)
        self.line_2.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_2.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_2.setObjectName("line_2")
        self.verticalLayout_5.addWidget(self.line_2)
        self.aggregates = QtWidgets.QComboBox(self.page)
        self.aggregates.setObjectName("aggregates")
        self.aggregates.addItem("")
        self.verticalLayout_5.addWidget(self.aggregates)
        self.functions = QtWidgets.QComboBox(self.page)
        self.functions.setObjectName("functions")
        self.functions.addItem("")
        self.verticalLayout_5.addWidget(self.functions)
        self.math = QtWidgets.QComboBox(self.page)
        self.math.setObjectName("math")
        self.math.addItem("")
        self.verticalLayout_5.addWidget(self.math)
        self.stringfct = QtWidgets.QComboBox(self.page)
        self.stringfct.setObjectName("stringfct")
        self.stringfct.addItem("")
        self.verticalLayout_5.addWidget(self.stringfct)
        self.operators = QtWidgets.QComboBox(self.page)
        self.operators.setObjectName("operators")
        self.operators.addItem("")
        self.verticalLayout_5.addWidget(self.operators)
        self.toolBox.addItem(self.page, "")
        self.page_2 = QtWidgets.QWidget()
        self.page_2.setGeometry(QtCore.QRect(0, 0, 154, 155))
        self.page_2.setObjectName("page_2")
        self.verticalLayout_6 = QtWidgets.QVBoxLayout(self.page_2)
        self.verticalLayout_6.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.columns_2 = QtWidgets.QComboBox(self.page_2)
        self.columns_2.setObjectName("columns_2")
        self.columns_2.addItem("")
        self.verticalLayout_6.addWidget(self.columns_2)
        self.extract = QtWidgets.QCheckBox(self.page_2)
        self.extract.setObjectName("extract")
        self.verticalLayout_6.addWidget(self.extract)
        self.values = QtWidgets.QListView(self.page_2)
        self.values.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.values.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.values.setProperty("showDropIndicator", False)
        self.values.setDragEnabled(False)
        self.values.setDragDropMode(QtWidgets.QAbstractItemView.NoDragDrop)
        self.values.setObjectName("values")
        self.verticalLayout_6.addWidget(self.values)
        self.toolBox.addItem(self.page_2, "")
        self.page_3 = QtWidgets.QWidget()
        self.page_3.setGeometry(QtCore.QRect(0, 0, 223, 122))
        self.page_3.setObjectName("page_3")
        self.verticalLayout_8 = QtWidgets.QVBoxLayout(self.page_3)
        self.verticalLayout_8.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_8.setObjectName("verticalLayout_8")
        self.table_idx = QtWidgets.QComboBox(self.page_3)
        self.table_idx.setObjectName("table_idx")
        self.table_idx.addItem("")
        self.verticalLayout_8.addWidget(self.table_idx)
        self.table_target = QtWidgets.QComboBox(self.page_3)
        self.table_target.setObjectName("table_target")
        self.table_target.addItem("")
        self.verticalLayout_8.addWidget(self.table_target)
        self.usertree = QtWidgets.QPushButton(self.page_3)
        self.usertree.setObjectName("usertree")
        self.verticalLayout_8.addWidget(self.usertree)
        spacerItem1 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_8.addItem(spacerItem1)
        self.toolBox.addItem(self.page_3, "")
        self.verticalLayout_4.addWidget(self.toolBox)
        self.horizontalLayout_6.addLayout(self.verticalLayout_4)
        self.verticalLayout_7.addLayout(self.horizontalLayout_6)
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.reset = QtWidgets.QPushButton(DbManagerQueryBuilderDlg)
        self.reset.setObjectName("reset")
        self.horizontalLayout_7.addWidget(self.reset)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_7.addItem(spacerItem2)
        self.buttonBox = QtWidgets.QDialogButtonBox(DbManagerQueryBuilderDlg)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setCenterButtons(True)
        self.buttonBox.setObjectName("buttonBox")
        self.horizontalLayout_7.addWidget(self.buttonBox)
        self.verticalLayout_7.addLayout(self.horizontalLayout_7)

        self.retranslateUi(DbManagerQueryBuilderDlg)
        self.toolBox.setCurrentIndex(0)
        self.buttonBox.rejected.connect(DbManagerQueryBuilderDlg.reject)
        self.buttonBox.accepted.connect(DbManagerQueryBuilderDlg.accept)
        self.reset.clicked.connect(self.where.clear)
        self.reset.clicked.connect(self.tab.clear)
        self.reset.clicked.connect(self.col.clear)
        self.reset.clicked.connect(self.columns.clear)
        QtCore.QMetaObject.connectSlotsByName(DbManagerQueryBuilderDlg)

    def retranslateUi(self, DbManagerQueryBuilderDlg):
        _translate = QtCore.QCoreApplication.translate
        DbManagerQueryBuilderDlg.setWindowTitle(_translate("DbManagerQueryBuilderDlg", "SQL Query Builder"))
        self.label.setText(_translate("DbManagerQueryBuilderDlg", "Columns"))
        self.label_2.setText(_translate("DbManagerQueryBuilderDlg", "Tables"))
        self.label_3.setText(_translate("DbManagerQueryBuilderDlg", "Where"))
        self.label_4.setText(_translate("DbManagerQueryBuilderDlg", "Group by"))
        self.label_5.setText(_translate("DbManagerQueryBuilderDlg", "Order by"))
        self.checkBox.setText(_translate("DbManagerQueryBuilderDlg", "Show system tables"))
        self.tables.setItemText(0, _translate("DbManagerQueryBuilderDlg", "Tables"))
        self.columns.setItemText(0, _translate("DbManagerQueryBuilderDlg", "Columns"))
        self.aggregates.setItemText(0, _translate("DbManagerQueryBuilderDlg", "Aggregates"))
        self.functions.setItemText(0, _translate("DbManagerQueryBuilderDlg", "Functions"))
        self.math.setItemText(0, _translate("DbManagerQueryBuilderDlg", "Math"))
        self.stringfct.setItemText(0, _translate("DbManagerQueryBuilderDlg", "Strings functions"))
        self.operators.setItemText(0, _translate("DbManagerQueryBuilderDlg", "Operators"))
        self.toolBox.setItemText(self.toolBox.indexOf(self.page), _translate("DbManagerQueryBuilderDlg", "Data"))
        self.columns_2.setItemText(0, _translate("DbManagerQueryBuilderDlg", "Columns"))
        self.extract.setText(_translate("DbManagerQueryBuilderDlg", "Only 10 first values"))
        self.toolBox.setItemText(self.toolBox.indexOf(self.page_2), _translate("DbManagerQueryBuilderDlg", "Columns\' values"))
        self.table_idx.setItemText(0, _translate("DbManagerQueryBuilderDlg", "Table (with spatial index)"))
        self.table_target.setItemText(0, _translate("DbManagerQueryBuilderDlg", "Table (Target)"))
        self.usertree.setText(_translate("DbManagerQueryBuilderDlg", "Use spatial index"))
        self.toolBox.setItemText(self.toolBox.indexOf(self.page_3), _translate("DbManagerQueryBuilderDlg", "Spatial index"))
        self.reset.setText(_translate("DbManagerQueryBuilderDlg", "&Reset"))

